'' =================================================================================================
''
''   File....... jm_xbee_api1.spin
''   Purpose.... Simple XBee API mode 1 coms management
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015-2017 Jon McPhalen
''               -- see below for terms of use                                                          
''   E-mail..... 
''   Started.... 
''   Updated.... 22 SEP 2017
''
'' =================================================================================================


con { fixed io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28


con

  BR_9600  =   9_615                                            ' corrected baud rates
  BR_19K2  =  19_230
  BR_38K4  =  38_461
  BR_57K6  =  58_823 
  BB_115K2 = 111_111

  BUF_SIZE = 64


obj

  time   : "jm_time_80"                                         '   timing & delays
  serial : "jm_fullduplexserial"                                ' * serial to xbee module


var

  long  cog                                                     ' running serial

  byte  txbuf[BUF_SIZE]                                         ' work buffers
  byte  rxbuf[BUF_SIZE]

  byte  rxlen                                                   ' length of packet in rx buffer

  
pub null

  ' This is not a top-level object

  
pub start(rx, tx)

'' Start XBee object on rx and tx pins
'' -- uses default baud rate

  startx(rx, tx, BR_9600)


pub startx(rx, tx, baud)

'' Start XBee object on rx and tx pins at specified baud
'' -- see baud table for corrected rates

  stop

  cog := serial.start(rx, tx, %0000, baud)                        ' start serial for xbee 
  time.pause(1)

  return cog


pub stop

'' Stops XBee cog if running          

  if (cog)
    serial.stop
    cog := 0
    

pub check_radio | response 

'' Checks for radio by reading MY address
'' -- returns MY id if found
'' -- returns -1 if no radio

  at_command(string("MY"))                                      ' request MY id

  return get_at_value


pub set_and_verify(p_str, value, len) 

'' Set parameter using 2-letter string
'' -- len must be 1, 2, or 4 (byte, word, long)
'' -- returns true if value set is read back

  set_value(p_str, value, len)                                  ' write value to xbee

  at_command(p_str)                                             ' request at value
   
  return (get_at_value == value)                                ' return true if good set

                
pub set_value(p_str, value, len) | idx, ofs

'' Set value using 2-letter AT string
'' -- len must be 1, 2, or 4 (byte, word, long)
'' -- no response packet

  txbuf[00] := $7E                                              ' delimiter
  txbuf[01] := $00                                              
  txbuf[02] := $00                                              ' length   
  txbuf[03] := $08                                              ' frame type (AT)
  txbuf[04] := $00                                              ' no auto response
  txbuf[05] := byte[p_str][0]                                   ' parameter string                                    
  txbuf[06] := byte[p_str][1]

  idx := 07                                                     ' start of data
  ofs := len-1                                                  ' ofset byte in value

  repeat len
    txbuf[idx++] := value.byte[ofs--]                           ' place value (Big Endian)

  txbuf[2] := idx-3                                             ' set packet len
  txbuf[idx] := calc_checksum(@txbuf[3], txbuf[2])              ' set checksum

  serial.rxflush                                                ' clear rx buf for new packet
  tx_frame(@txbuf, len + 8)                                     ' header + payload + checksum                                      
  serial.txflush                                                ' let it finish


pub at_command(p_str)

'' Write at command string
'' -- generates response packet

  txbuf[00] := $7E                                              ' delimiter
  txbuf[01] := $00                                              
  txbuf[02] := $04                                              ' length   
  txbuf[03] := $08                                              ' frame type (AT)
  txbuf[04] := $01                                              ' frame id (for response)
  txbuf[05] := byte[p_str][0]                                   ' parameter string 
  txbuf[06] := byte[p_str][1]
  txbuf[07] := calc_checksum(@txbuf[3], 4)                      ' set checksum   

  serial.rxflush                                                ' clear rx buf for new packet
  tx_frame(@txbuf, 8)                                           ' send the packet
  serial.txflush                                                ' let it finish


pub tx_frame(p_frame, len)

'' Transmit frame
'' -- transmits data in buffer at p_frame

  repeat len                                                    ' send the frame
    serial.tx(byte[p_frame++])


pub get_rx_frame(ms) : len | c, pb

'' Check for and return rx frame if available

  if (ms =< 0)
    c := serial.rxcheck
  else
    c := serial.rxtime(ms)

  if (c == $7E)                                                 ' frame header?
    bytefill(@rxbuf, 0, BUF_SIZE)                               ' clear input buffer 
    rxbuf[len++] := c

    c := serial.rxtime(1)                                       ' msb of payload (always 0)  
    if (c < 0)               
      return 0               
    else    
      rxbuf[len++] := c

    pb := serial.rxtime(1)                                      ' lsb of payload bytes    
    if (pb < 0)               
      return 0               
    else    
      rxbuf[len++] := pb
        
    repeat (pb+1)                                               ' get payload bytes + check sum
      c := serial.rxtime(1)
      if (c < 0)  
        return 0  
      else
        rxbuf[len++] := c


pub validate_rx_frame | last

'' Return true if last rx frame is valid

  last := rxbuf[2] + 3                                          ' location of rx'd checksum

  if (calc_checksum(@rxbuf[3], rxbuf[2]) == rxbuf[last])        ' checksum match?
    return true
  else
    return false


pub get_at_value

'' Get value of last requested  (via AT) parameter
'' -- returns -1 if no value returned

  if (get_rx_frame(25))                                         ' give xbee some time
    if (validate_rx_frame)                                      ' response detected?
      if ((rxbuf[3] == $88) and (rxbuf[7] == $00))              ' good AT response frame?
        return extract_at_value                                 ' extract value from packet  
     
  return -1                                                     ' no response....
  

pri extract_at_value | len, ofs, idx

'' Get value of parameter from rx stream
'' -- stream is response to AT parameter request

  len := rxbuf[2] - 5                                           ' determine parameter bytes
  idx := 8                                                      ' starting position in packet
  ofs := len - 1                                                ' calc byte offset

  repeat len
    result.byte[ofs--] := rxbuf[idx++]                          ' extract parameter


pub calc_checksum(p_buf, len) : cs
                       
  repeat len       
    cs += byte[p_buf++]                                         ' accumulate packet bytes           

  cs := $FF - cs.byte[0]                                        ' finalize checksum}

 
pub rx_count

  return rxlen
  

pub get_rx(idx)

'' Return a byte from the last rx stream

  if (idx < BUF_SIZE)
    return rxbuf[idx]
  else
    return -1


pub flush

'' Flush serial rx buffer

  serial.rxflush


pub tx_addr

  return @txbuf


pub rx_addr

  return @rxbuf


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}       